@@ -16,20 +16,26 @@ import six
 import pytest
 from _pytest.compat import CaptureIO
 
-
-patchsysdict = {0: 'stdin', 1: 'stdout', 2: 'stderr'}
+patchsysdict = {0: "stdin", 1: "stdout", 2: "stderr"}
 
 
 def pytest_addoption(parser):
     group = parser.getgroup("general")
     group._addoption(
-        '--capture', action="store",
+        "--capture",
+        action="store",
         default="fd" if hasattr(os, "dup") else "sys",
-        metavar="method", choices=['fd', 'sys', 'no'],
-        help="per-test capturing method: one of fd|sys|no.")
+        metavar="method",
+        choices=["fd", "sys", "no"],
+        help="per-test capturing method: one of fd|sys|no.",
+    )
     group._addoption(
-        '-s', action="store_const", const="no", dest="capture",
-        help="shortcut for --capture=no.")
+        "-s",
+        action="store_const",
+        const="no",
+        dest="capture",
+        help="shortcut for --capture=no.",
+    )
 
 
 @pytest.hookimpl(hookwrapper=True)
@@ -50,13 +56,15 @@ def pytest_load_initial_conftests(early_config, parser, args):
     def silence_logging_at_shutdown():
         if "logging" in sys.modules:
             sys.modules["logging"].raiseExceptions = False
+
     early_config.add_cleanup(silence_logging_at_shutdown)
 
     # finally trigger conftest loading but while capturing (issue93)
     capman.start_global_capturing()
     outcome = yield
-    out, err = capman.suspend_global_capture()
+    capman.suspend_global_capture()
     if outcome.excinfo is not None:
+        out, err = capman.read_global_capture()
         sys.stdout.write(out)
         sys.stderr.write(err)
 
@@ -77,6 +85,7 @@ class CaptureManager(object):
     def __init__(self, method):
         self._method = method
         self._global_capturing = None
+        self._current_item = None
 
     def _getcapture(self, method):
         if method == "fd":
@@ -88,6 +97,8 @@ class CaptureManager(object):
         else:
             raise ValueError("unknown capturing method: %r" % method)
 
+    # Global capturing control
+
     def start_global_capturing(self):
         assert self._global_capturing is None
         self._global_capturing = self._getcapture(self._method)
@@ -102,16 +113,15 @@ class CaptureManager(object):
     def resume_global_capture(self):
         self._global_capturing.resume_capturing()
 
-    def suspend_global_capture(self, item=None, in_=False):
-        if item is not None:
-            self.deactivate_fixture(item)
+    def suspend_global_capture(self, in_=False):
         cap = getattr(self, "_global_capturing", None)
         if cap is not None:
-            try:
-                outerr = cap.readouterr()
-            finally:
-                cap.suspend_capturing(in_=in_)
-            return outerr
+            cap.suspend_capturing(in_=in_)
+
+    def read_global_capture(self):
+        return self._global_capturing.readouterr()
+
+    # Fixture Control (its just forwarding, think about removing this later)
 
     def activate_fixture(self, item):
         """If the current item is using ``capsys`` or ``capfd``, activate them so they take precedence over
@@ -127,12 +137,53 @@ class CaptureManager(object):
         if fixture is not None:
             fixture.close()
 
+    def suspend_fixture(self, item):
+        fixture = getattr(item, "_capture_fixture", None)
+        if fixture is not None:
+            fixture._suspend()
+
+    def resume_fixture(self, item):
+        fixture = getattr(item, "_capture_fixture", None)
+        if fixture is not None:
+            fixture._resume()
+
+    # Helper context managers
+
+    @contextlib.contextmanager
+    def global_and_fixture_disabled(self):
+        """Context manager to temporarily disables global and current fixture capturing."""
+        # Need to undo local capsys-et-al if exists before disabling global capture
+        self.suspend_fixture(self._current_item)
+        self.suspend_global_capture(in_=False)
+        try:
+            yield
+        finally:
+            self.resume_global_capture()
+            self.resume_fixture(self._current_item)
+
+    @contextlib.contextmanager
+    def item_capture(self, when, item):
+        self.resume_global_capture()
+        self.activate_fixture(item)
+        try:
+            yield
+        finally:
+            self.deactivate_fixture(item)
+            self.suspend_global_capture(in_=False)
+
+        out, err = self.read_global_capture()
+        item.add_report_section(when, "stdout", out)
+        item.add_report_section(when, "stderr", err)
+
+    # Hooks
+
     @pytest.hookimpl(hookwrapper=True)
     def pytest_make_collect_report(self, collector):
         if isinstance(collector, pytest.File):
             self.resume_global_capture()
             outcome = yield
-            out, err = self.suspend_global_capture()
+            self.suspend_global_capture()
+            out, err = self.read_global_capture()
             rep = outcome.get_result()
             if out:
                 rep.sections.append(("Captured stdout", out))
@@ -142,29 +193,25 @@ class CaptureManager(object):
             yield
 
     @pytest.hookimpl(hookwrapper=True)
-    def pytest_runtest_setup(self, item):
-        self.resume_global_capture()
-        # no need to activate a capture fixture because they activate themselves during creation; this
-        # only makes sense when a fixture uses a capture fixture, otherwise the capture fixture will
-        # be activated during pytest_runtest_call
+    def pytest_runtest_protocol(self, item):
+        self._current_item = item
         yield
-        self.suspend_capture_item(item, "setup")
+        self._current_item = None
+
+    @pytest.hookimpl(hookwrapper=True)
+    def pytest_runtest_setup(self, item):
+        with self.item_capture("setup", item):
+            yield
 
     @pytest.hookimpl(hookwrapper=True)
     def pytest_runtest_call(self, item):
-        self.resume_global_capture()
-        # it is important to activate this fixture during the call phase so it overwrites the "global"
-        # capture
-        self.activate_fixture(item)
-        yield
-        self.suspend_capture_item(item, "call")
+        with self.item_capture("call", item):
+            yield
 
     @pytest.hookimpl(hookwrapper=True)
     def pytest_runtest_teardown(self, item):
-        self.resume_global_capture()
-        self.activate_fixture(item)
-        yield
-        self.suspend_capture_item(item, "teardown")
+        with self.item_capture("teardown", item):
+            yield
 
     @pytest.hookimpl(tryfirst=True)
     def pytest_keyboard_interrupt(self, excinfo):
@@ -174,24 +221,17 @@ class CaptureManager(object):
     def pytest_internalerror(self, excinfo):
         self.stop_global_capturing()
 
-    def suspend_capture_item(self, item, when, in_=False):
-        out, err = self.suspend_global_capture(item, in_=in_)
-        item.add_report_section(when, "stdout", out)
-        item.add_report_section(when, "stderr", err)
 
-
-capture_fixtures = {'capfd', 'capfdbinary', 'capsys', 'capsysbinary'}
+capture_fixtures = {"capfd", "capfdbinary", "capsys", "capsysbinary"}
 
 
 def _ensure_only_one_capture_fixture(request, name):
-    fixtures = set(request.fixturenames) & capture_fixtures - set((name,))
+    fixtures = set(request.fixturenames) & capture_fixtures - {name}
     if fixtures:
         fixtures = sorted(fixtures)
         fixtures = fixtures[0] if len(fixtures) == 1 else fixtures
         raise request.raiseerror(
-            "cannot use {0} and {1} at the same time".format(
-                fixtures, name,
-            ),
+            "cannot use {} and {} at the same time".format(fixtures, name)
         )
 
 
@@ -202,7 +242,7 @@ def capsys(request):
     which return a ``(out, err)`` namedtuple.  ``out`` and ``err`` will be ``text``
     objects.
     """
-    _ensure_only_one_capture_fixture(request, 'capsys')
+    _ensure_only_one_capture_fixture(request, "capsys")
     with _install_capture_fixture_on_item(request, SysCapture) as fixture:
         yield fixture
 
@@ -214,11 +254,11 @@ def capsysbinary(request):
     which return a ``(out, err)`` tuple.  ``out`` and ``err`` will be ``bytes``
     objects.
     """
-    _ensure_only_one_capture_fixture(request, 'capsysbinary')
+    _ensure_only_one_capture_fixture(request, "capsysbinary")
     # Currently, the implementation uses the python3 specific `.buffer`
     # property of CaptureIO.
     if sys.version_info < (3,):
-        raise request.raiseerror('capsysbinary is only supported on python 3')
+        raise request.raiseerror("capsysbinary is only supported on python 3")
     with _install_capture_fixture_on_item(request, SysCaptureBinary) as fixture:
         yield fixture
 
@@ -230,9 +270,11 @@ def capfd(request):
     which return a ``(out, err)`` tuple.  ``out`` and ``err`` will be ``text``
     objects.
     """
-    _ensure_only_one_capture_fixture(request, 'capfd')
-    if not hasattr(os, 'dup'):
-        pytest.skip("capfd fixture needs os.dup function which is not available in this system")
+    _ensure_only_one_capture_fixture(request, "capfd")
+    if not hasattr(os, "dup"):
+        pytest.skip(
+            "capfd fixture needs os.dup function which is not available in this system"
+        )
     with _install_capture_fixture_on_item(request, FDCapture) as fixture:
         yield fixture
 
@@ -244,9 +286,11 @@ def capfdbinary(request):
     which return a ``(out, err)`` tuple.  ``out`` and ``err`` will be
     ``bytes`` objects.
     """
-    _ensure_only_one_capture_fixture(request, 'capfdbinary')
-    if not hasattr(os, 'dup'):
-        pytest.skip("capfdbinary fixture needs os.dup function which is not available in this system")
+    _ensure_only_one_capture_fixture(request, "capfdbinary")
+    if not hasattr(os, "dup"):
+        pytest.skip(
+            "capfdbinary fixture needs os.dup function which is not available in this system"
+        )
     with _install_capture_fixture_on_item(request, FDCaptureBinary) as fixture:
         yield fixture
 
@@ -261,7 +305,7 @@ def _install_capture_fixture_on_item(request, capture_class):
     by ``CaptureManager`` during its ``pytest_runtest_*`` hooks.
     """
     request.node._capture_fixture = fixture = CaptureFixture(capture_class, request)
-    capmanager = request.config.pluginmanager.getplugin('capturemanager')
+    capmanager = request.config.pluginmanager.getplugin("capturemanager")
     # need to active this fixture right away in case it is being used by another fixture (setup phase)
     # if this fixture is being used only by a test function (call phase), then we wouldn't need this
     # activation, but it doesn't hurt
@@ -276,46 +320,62 @@ class CaptureFixture(object):
     Object returned by :py:func:`capsys`, :py:func:`capsysbinary`, :py:func:`capfd` and :py:func:`capfdbinary`
     fixtures.
     """
+
     def __init__(self, captureclass, request):
         self.captureclass = captureclass
         self.request = request
+        self._capture = None
+        self._captured_out = self.captureclass.EMPTY_BUFFER
+        self._captured_err = self.captureclass.EMPTY_BUFFER
 
     def _start(self):
-        self._capture = MultiCapture(out=True, err=True, in_=False,
-                                     Capture=self.captureclass)
-        self._capture.start_capturing()
+        # Start if not started yet
+        if getattr(self, "_capture", None) is None:
+            self._capture = MultiCapture(
+                out=True, err=True, in_=False, Capture=self.captureclass
+            )
+            self._capture.start_capturing()
 
     def close(self):
-        cap = self.__dict__.pop("_capture", None)
-        if cap is not None:
-            self._outerr = cap.pop_outerr_to_orig()
-            cap.stop_capturing()
+        if self._capture is not None:
+            out, err = self._capture.pop_outerr_to_orig()
+            self._captured_out += out
+            self._captured_err += err
+            self._capture.stop_capturing()
+            self._capture = None
 
     def readouterr(self):
         """Read and return the captured output so far, resetting the internal buffer.
 
         :return: captured content as a namedtuple with  ``out`` and ``err`` string attributes
         """
-        try:
-            return self._capture.readouterr()
-        except AttributeError:
-            return self._outerr
+        captured_out, captured_err = self._captured_out, self._captured_err
+        if self._capture is not None:
+            out, err = self._capture.readouterr()
+            captured_out += out
+            captured_err += err
+        self._captured_out = self.captureclass.EMPTY_BUFFER
+        self._captured_err = self.captureclass.EMPTY_BUFFER
+        return CaptureResult(captured_out, captured_err)
+
+    def _suspend(self):
+        """Suspends this fixture's own capturing temporarily."""
+        self._capture.suspend_capturing()
+
+    def _resume(self):
+        """Resumes this fixture's own capturing temporarily."""
+        self._capture.resume_capturing()
 
     @contextlib.contextmanager
     def disabled(self):
         """Temporarily disables capture while inside the 'with' block."""
-        self._capture.suspend_capturing()
-        capmanager = self.request.config.pluginmanager.getplugin('capturemanager')
-        capmanager.suspend_global_capture(item=None, in_=False)
-        try:
+        capmanager = self.request.config.pluginmanager.getplugin("capturemanager")
+        with capmanager.global_and_fixture_disabled():
             yield
-        finally:
-            capmanager.resume_global_capture()
-            self._capture.resume_capturing()
 
 
 def safe_text_dupfile(f, mode, default_encoding="UTF8"):
-    """ return a open text file object that's a duplicate of f on the
+    """ return an open text file object that's a duplicate of f on the
         FD-level if possible.
     """
     encoding = getattr(f, "encoding", None)
@@ -346,7 +406,7 @@ class EncodedFile(object):
         self.buffer.write(obj)
 
     def writelines(self, linelist):
-        data = ''.join(linelist)
+        data = "".join(linelist)
         self.write(data)
 
     @property
@@ -409,7 +469,7 @@ class MultiCapture(object):
 
     def stop_capturing(self):
         """ stop capturing and reset capturing streams """
-        if hasattr(self, '_reset'):
+        if hasattr(self, "_reset"):
             raise ValueError("was already stopped")
         self._reset = True
         if self.out:
@@ -421,11 +481,14 @@ class MultiCapture(object):
 
     def readouterr(self):
         """ return snapshot unicode value of stdout/stderr capturings. """
-        return CaptureResult(self.out.snap() if self.out is not None else "",
-                             self.err.snap() if self.err is not None else "")
+        return CaptureResult(
+            self.out.snap() if self.out is not None else "",
+            self.err.snap() if self.err is not None else "",
+        )
 
 
 class NoCapture(object):
+    EMPTY_BUFFER = None
     __init__ = start = done = suspend = resume = lambda *args: None
 
 
@@ -435,6 +498,8 @@ class FDCaptureBinary(object):
     snap() produces `bytes`
     """
 
+    EMPTY_BUFFER = bytes()
+
     def __init__(self, targetfd, tmpfile=None):
         self.targetfd = targetfd
         try:
@@ -507,6 +572,9 @@ class FDCapture(FDCaptureBinary):
 
     snap() produces text
     """
+
+    EMPTY_BUFFER = str()
+
     def snap(self):
         res = FDCaptureBinary.snap(self)
         enc = getattr(self.tmpfile, "encoding", None)
@@ -516,6 +584,9 @@ class FDCapture(FDCaptureBinary):
 
 
 class SysCapture(object):
+
+    EMPTY_BUFFER = str()
+
     def __init__(self, fd, tmpfile=None):
         name = patchsysdict[fd]
         self._old = getattr(sys, name)
@@ -553,6 +624,8 @@ class SysCapture(object):
 
 
 class SysCaptureBinary(SysCapture):
+    EMPTY_BUFFER = bytes()
+
     def snap(self):
         res = self.tmpfile.buffer.getvalue()
         self.tmpfile.seek(0)
@@ -572,6 +645,7 @@ class DontReadFromInput(six.Iterator):
 
     def read(self, *args):
         raise IOError("reading from stdin while output is captured")
+
     readline = read
     readlines = read
     __next__ = read
@@ -580,8 +654,7 @@ class DontReadFromInput(six.Iterator):
         return self
 
     def fileno(self):
-        raise UnsupportedOperation("redirected stdin is pseudofile, "
-                                   "has no fileno()")
+        raise UnsupportedOperation("redirected stdin is pseudofile, has no fileno()")
 
     def isatty(self):
         return False
@@ -594,7 +667,7 @@ class DontReadFromInput(six.Iterator):
         if sys.version_info >= (3, 0):
             return self
         else:
-            raise AttributeError('redirected stdin has no attribute buffer')
+            raise AttributeError("redirected stdin has no attribute buffer")
 
 
 def _colorama_workaround():
@@ -607,7 +680,7 @@ def _colorama_workaround():
     fail in various ways.
     """
 
-    if not sys.platform.startswith('win32'):
+    if not sys.platform.startswith("win32"):
         return
     try:
         import colorama  # noqa
@@ -634,7 +707,7 @@ def _readline_workaround():
     See https://github.com/pytest-dev/pytest/pull/1281
     """
 
-    if not sys.platform.startswith('win32'):
+    if not sys.platform.startswith("win32"):
         return
     try:
         import readline  # noqa
@@ -664,21 +737,21 @@ def _py36_windowsconsoleio_workaround(stream):
 
     See https://github.com/pytest-dev/py/issues/103
     """
-    if not sys.platform.startswith('win32') or sys.version_info[:2] < (3, 6):
+    if not sys.platform.startswith("win32") or sys.version_info[:2] < (3, 6):
         return
 
     # bail out if ``stream`` doesn't seem like a proper ``io`` stream (#2666)
-    if not hasattr(stream, 'buffer'):
+    if not hasattr(stream, "buffer"):
         return
 
-    buffered = hasattr(stream.buffer, 'raw')
+    buffered = hasattr(stream.buffer, "raw")
     raw_stdout = stream.buffer.raw if buffered else stream.buffer
 
     if not isinstance(raw_stdout, io._WindowsConsoleIO):
         return
 
     def _reopen_stdio(f, mode):
-        if not buffered and mode[0] == 'w':
+        if not buffered and mode[0] == "w":
             buffering = 0
         else:
             buffering = -1
@@ -688,11 +761,12 @@ def _py36_windowsconsoleio_workaround(stream):
             f.encoding,
             f.errors,
             f.newlines,
-            f.line_buffering)
+            f.line_buffering,
+        )
 
-    sys.__stdin__ = sys.stdin = _reopen_stdio(sys.stdin, 'rb')
-    sys.__stdout__ = sys.stdout = _reopen_stdio(sys.stdout, 'wb')
-    sys.__stderr__ = sys.stderr = _reopen_stdio(sys.stderr, 'wb')
+    sys.__stdin__ = sys.stdin = _reopen_stdio(sys.stdin, "rb")
+    sys.__stdout__ = sys.stdout = _reopen_stdio(sys.stdout, "wb")
+    sys.__stderr__ = sys.stderr = _reopen_stdio(sys.stderr, "wb")
 
 
 def _attempt_to_close_capture_file(f):
