@@ -38,16 +38,15 @@ class Junit(py.xml.Namespace):
 # this dynamically instead of hardcoding it.  The spec range of valid
 # chars is: Char ::= #x9 | #xA | #xD | [#x20-#xD7FF] | [#xE000-#xFFFD]
 #                    | [#x10000-#x10FFFF]
-_legal_chars = (0x09, 0x0A, 0x0d)
-_legal_ranges = (
-    (0x20, 0x7E), (0x80, 0xD7FF), (0xE000, 0xFFFD), (0x10000, 0x10FFFF),
-)
+_legal_chars = (0x09, 0x0A, 0x0D)
+_legal_ranges = ((0x20, 0x7E), (0x80, 0xD7FF), (0xE000, 0xFFFD), (0x10000, 0x10FFFF))
 _legal_xml_re = [
     unicode("%s-%s") % (unichr(low), unichr(high))
-    for (low, high) in _legal_ranges if low < sys.maxunicode
+    for (low, high) in _legal_ranges
+    if low < sys.maxunicode
 ]
 _legal_xml_re = [unichr(x) for x in _legal_chars] + _legal_xml_re
-illegal_xml_re = re.compile(unicode('[^%s]') % unicode('').join(_legal_xml_re))
+illegal_xml_re = re.compile(unicode("[^%s]") % unicode("").join(_legal_xml_re))
 del _legal_chars
 del _legal_ranges
 del _legal_xml_re
@@ -59,9 +58,9 @@ def bin_xml_escape(arg):
     def repl(matchobj):
         i = ord(matchobj.group())
         if i <= 0xFF:
-            return unicode('#x%02X') % i
+            return unicode("#x%02X") % i
         else:
-            return unicode('#x%04X') % i
+            return unicode("#x%04X") % i
 
     return py.xml.raw(illegal_xml_re.sub(repl, py.xml.escape(arg)))
 
@@ -92,11 +91,13 @@ class _NodeReporter(object):
         """Return a Junit node containing custom properties, if any.
         """
         if self.properties:
-            return Junit.properties([
-                Junit.property(name=name, value=value)
-                for name, value in self.properties
-            ])
-        return ''
+            return Junit.properties(
+                [
+                    Junit.property(name=name, value=value)
+                    for name, value in self.properties
+                ]
+            )
+        return ""
 
     def record_testreport(self, testreport):
         assert not self.testcase
@@ -135,53 +136,57 @@ class _NodeReporter(object):
         content_err = report.capstderr
 
         if content_log or content_out:
-            if content_log and self.xml.logging == 'system-out':
+            if content_log and self.xml.logging == "system-out":
                 if content_out:
                     # syncing stdout and the log-output is not done yet. It's
                     # probably not worth the effort. Therefore, first the captured
                     # stdout is shown and then the captured logs.
-                    content = '\n'.join([
-                        ' Captured Stdout '.center(80, '-'),
-                        content_out,
-                        '',
-                        ' Captured Log '.center(80, '-'),
-                        content_log])
+                    content = "\n".join(
+                        [
+                            " Captured Stdout ".center(80, "-"),
+                            content_out,
+                            "",
+                            " Captured Log ".center(80, "-"),
+                            content_log,
+                        ]
+                    )
                 else:
                     content = content_log
             else:
                 content = content_out
 
             if content:
-                tag = getattr(Junit, 'system-out')
+                tag = getattr(Junit, "system-out")
                 self.append(tag(bin_xml_escape(content)))
 
         if content_log or content_err:
-            if content_log and self.xml.logging == 'system-err':
+            if content_log and self.xml.logging == "system-err":
                 if content_err:
-                    content = '\n'.join([
-                        ' Captured Stderr '.center(80, '-'),
-                        content_err,
-                        '',
-                        ' Captured Log '.center(80, '-'),
-                        content_log])
+                    content = "\n".join(
+                        [
+                            " Captured Stderr ".center(80, "-"),
+                            content_err,
+                            "",
+                            " Captured Log ".center(80, "-"),
+                            content_log,
+                        ]
+                    )
                 else:
                     content = content_log
             else:
                 content = content_err
 
             if content:
-                tag = getattr(Junit, 'system-err')
+                tag = getattr(Junit, "system-err")
                 self.append(tag(bin_xml_escape(content)))
 
     def append_pass(self, report):
-        self.add_stats('passed')
+        self.add_stats("passed")
 
     def append_failure(self, report):
         # msg = str(report.longrepr.reprtraceback.extraline)
         if hasattr(report, "wasxfail"):
-            self._add_simple(
-                Junit.skipped,
-                "xfail-marked test passes unexpectedly")
+            self._add_simple(Junit.skipped, "xfail-marked test passes unexpectedly")
         else:
             if hasattr(report.longrepr, "reprcrash"):
                 message = report.longrepr.reprcrash.message
@@ -196,35 +201,37 @@ class _NodeReporter(object):
 
     def append_collect_error(self, report):
         # msg = str(report.longrepr.reprtraceback.extraline)
-        self.append(Junit.error(bin_xml_escape(report.longrepr),
-                                message="collection failure"))
+        self.append(
+            Junit.error(bin_xml_escape(report.longrepr), message="collection failure")
+        )
 
     def append_collect_skipped(self, report):
-        self._add_simple(
-            Junit.skipped, "collection skipped", report.longrepr)
+        self._add_simple(Junit.skipped, "collection skipped", report.longrepr)
 
     def append_error(self, report):
-        if getattr(report, 'when', None) == 'teardown':
+        if getattr(report, "when", None) == "teardown":
             msg = "test teardown failure"
         else:
             msg = "test setup failure"
-        self._add_simple(
-            Junit.error, msg, report.longrepr)
+        self._add_simple(Junit.error, msg, report.longrepr)
 
     def append_skipped(self, report):
         if hasattr(report, "wasxfail"):
-            self._add_simple(
-                Junit.skipped, "expected test failure", report.wasxfail
-            )
+            self._add_simple(Junit.skipped, "expected test failure", report.wasxfail)
         else:
             filename, lineno, skipreason = report.longrepr
             if skipreason.startswith("Skipped: "):
-                skipreason = bin_xml_escape(skipreason[9:])
+                skipreason = skipreason[9:]
+            details = "%s:%s: %s" % (filename, lineno, skipreason)
+
             self.append(
-                Junit.skipped("%s:%s: %s" % (filename, lineno, skipreason),
-                              type="pytest.skip",
-                              message=skipreason))
-        self.write_captured_output(report)
+                Junit.skipped(
+                    bin_xml_escape(details),
+                    type="pytest.skip",
+                    message=bin_xml_escape(skipreason),
+                )
+            )
+            self.write_captured_output(report)
 
     def finalize(self):
         data = self.to_xml().unicode(indent=0)
@@ -245,21 +252,19 @@ def record_property(request):
         def test_function(record_property):
             record_property("example_key", 1)
     """
+
     def append_property(name, value):
         request.node.user_properties.append((name, value))
+
     return append_property
 
 
 @pytest.fixture
-def record_xml_property(record_property):
+def record_xml_property(record_property, request):
     """(Deprecated) use record_property."""
-    import warnings
     from _pytest import deprecated
-    warnings.warn(
-        deprecated.RECORD_XML_PROPERTY,
-        DeprecationWarning,
-        stacklevel=2
-    )
+
+    request.node.warn(deprecated.RECORD_XML_PROPERTY)
 
     return record_property
 
@@ -270,15 +275,15 @@ def record_xml_attribute(request):
     The fixture is callable with ``(name, value)``, with value being
     automatically xml-encoded
     """
-    request.node.warn(
-        code='C3',
-        message='record_xml_attribute is an experimental feature',
-    )
+    from _pytest.warning_types import PytestWarning
+
+    request.node.warn(PytestWarning("record_xml_attribute is an experimental feature"))
     xml = getattr(request.config, "_xml", None)
     if xml is not None:
         node_reporter = xml.node_reporter(request.node.nodeid)
         return node_reporter.add_attribute
     else:
+
         def add_attr_noop(name, value):
             pass
 
@@ -288,51 +293,63 @@ def record_xml_attribute(request):
 def pytest_addoption(parser):
     group = parser.getgroup("terminal reporting")
     group.addoption(
-        '--junitxml', '--junit-xml',
+        "--junitxml",
+        "--junit-xml",
         action="store",
         dest="xmlpath",
         metavar="path",
         type=functools.partial(filename_arg, optname="--junitxml"),
         default=None,
-        help="create junit-xml style report file at given path.")
+        help="create junit-xml style report file at given path.",
+    )
     group.addoption(
-        '--junitprefix', '--junit-prefix',
+        "--junitprefix",
+        "--junit-prefix",
         action="store",
         metavar="str",
         default=None,
-        help="prepend prefix to classnames in junit-xml output")
-    parser.addini("junit_suite_name", "Test suite name for JUnit report", default="pytest")
-    parser.addini("junit_logging", "Write captured log messages to JUnit report: "
-                  "one of no|system-out|system-err",
-                  default="no")  # choices=['no', 'stdout', 'stderr'])
+        help="prepend prefix to classnames in junit-xml output",
+    )
+    parser.addini(
+        "junit_suite_name", "Test suite name for JUnit report", default="pytest"
+    )
+    parser.addini(
+        "junit_logging",
+        "Write captured log messages to JUnit report: "
+        "one of no|system-out|system-err",
+        default="no",
+    )  # choices=['no', 'stdout', 'stderr'])
 
 
 def pytest_configure(config):
     xmlpath = config.option.xmlpath
     # prevent opening xmllog on slave nodes (xdist)
-    if xmlpath and not hasattr(config, 'slaveinput'):
-        config._xml = LogXML(xmlpath, config.option.junitprefix,
-                             config.getini("junit_suite_name"),
-                             config.getini("junit_logging"))
+    if xmlpath and not hasattr(config, "slaveinput"):
+        config._xml = LogXML(
+            xmlpath,
+            config.option.junitprefix,
+            config.getini("junit_suite_name"),
+            config.getini("junit_logging"),
+        )
         config.pluginmanager.register(config._xml)
 
 
 def pytest_unconfigure(config):
-    xml = getattr(config, '_xml', None)
+    xml = getattr(config, "_xml", None)
     if xml:
         del config._xml
         config.pluginmanager.unregister(xml)
 
 
 def mangle_test_address(address):
-    path, possible_open_bracket, params = address.partition('[')
+    path, possible_open_bracket, params = address.partition("[")
     names = path.split("::")
     try:
-        names.remove('()')
+        names.remove("()")
     except ValueError:
         pass
     # convert file path to dotted path
-    names[0] = names[0].replace(nodes.SEP, '.')
+    names[0] = names[0].replace(nodes.SEP, ".")
     names[0] = _py_ext_re.sub("", names[0])
     # put any params back
     names[-1] += possible_open_bracket + params
@@ -346,12 +363,7 @@ class LogXML(object):
         self.prefix = prefix
         self.suite_name = suite_name
         self.logging = logging
-        self.stats = dict.fromkeys([
-            'error',
-            'passed',
-            'failure',
-            'skipped',
-        ], 0)
+        self.stats = dict.fromkeys(["error", "passed", "failure", "skipped"], 0)
         self.node_reporters = {}  # nodeid -> _NodeReporter
         self.node_reporters_ordered = []
         self.global_properties = []
@@ -360,17 +372,17 @@ class LogXML(object):
         self.cnt_double_fail_tests = 0
 
     def finalize(self, report):
-        nodeid = getattr(report, 'nodeid', report)
+        nodeid = getattr(report, "nodeid", report)
         # local hack to handle xdist report order
-        slavenode = getattr(report, 'node', None)
+        slavenode = getattr(report, "node", None)
         reporter = self.node_reporters.pop((nodeid, slavenode))
         if reporter is not None:
             reporter.finalize()
 
     def node_reporter(self, report):
-        nodeid = getattr(report, 'nodeid', report)
+        nodeid = getattr(report, "nodeid", report)
         # local hack to handle xdist report order
-        slavenode = getattr(report, 'node', None)
+        slavenode = getattr(report, "node", None)
 
         key = nodeid, slavenode
 
@@ -428,12 +440,17 @@ class LogXML(object):
                 report_wid = getattr(report, "worker_id", None)
                 report_ii = getattr(report, "item_index", None)
                 close_report = next(
-                    (rep for rep in self.open_reports
-                     if (rep.nodeid == report.nodeid and
-                         getattr(rep, "item_index", None) == report_ii and
-                         getattr(rep, "worker_id", None) == report_wid
-                         )
-                     ), None)
+                    (
+                        rep
+                        for rep in self.open_reports
+                        if (
+                            rep.nodeid == report.nodeid
+                            and getattr(rep, "item_index", None) == report_ii
+                            and getattr(rep, "worker_id", None) == report_wid
+                        )
+                    ),
+                    None,
+                )
                 if close_report:
                     # We need to open new testcase in case we have failure in
                     # call and error in teardown in order to follow junit
@@ -461,12 +478,17 @@ class LogXML(object):
             report_wid = getattr(report, "worker_id", None)
             report_ii = getattr(report, "item_index", None)
             close_report = next(
-                (rep for rep in self.open_reports
-                 if (rep.nodeid == report.nodeid and
-                     getattr(rep, "item_index", None) == report_ii and
-                     getattr(rep, "worker_id", None) == report_wid
-                     )
-                 ), None)
+                (
+                    rep
+                    for rep in self.open_reports
+                    if (
+                        rep.nodeid == report.nodeid
+                        and getattr(rep, "item_index", None) == report_ii
+                        and getattr(rep, "worker_id", None) == report_wid
+                    )
+                ),
+                None,
+            )
             if close_report:
                 self.open_reports.remove(close_report)
 
@@ -475,7 +497,7 @@ class LogXML(object):
         the Junit.testcase with the new total if already created.
         """
         reporter = self.node_reporter(report)
-        reporter.duration += getattr(report, 'duration', 0.0)
+        reporter.duration += getattr(report, "duration", 0.0)
 
     def pytest_collectreport(self, report):
         if not report.passed:
@@ -486,9 +508,9 @@ class LogXML(object):
                 reporter.append_collect_skipped(report)
 
     def pytest_internalerror(self, excrepr):
-        reporter = self.node_reporter('internal')
-        reporter.attrs.update(classname="pytest", name='internal')
-        reporter._add_simple(Junit.error, 'internal error', excrepr)
+        reporter = self.node_reporter("internal")
+        reporter.attrs.update(classname="pytest", name="internal")
+        reporter._add_simple(Junit.error, "internal error", excrepr)
 
     def pytest_sessionstart(self):
         self.suite_start_time = time.time()
@@ -497,29 +519,35 @@ class LogXML(object):
         dirname = os.path.dirname(os.path.abspath(self.logfile))
         if not os.path.isdir(dirname):
             os.makedirs(dirname)
-        logfile = open(self.logfile, 'w', encoding='utf-8')
+        logfile = open(self.logfile, "w", encoding="utf-8")
         suite_stop_time = time.time()
         suite_time_delta = suite_stop_time - self.suite_start_time
 
-        numtests = (self.stats['passed'] + self.stats['failure'] +
-                    self.stats['skipped'] + self.stats['error'] -
-                    self.cnt_double_fail_tests)
+        numtests = (
+            self.stats["passed"]
+            + self.stats["failure"]
+            + self.stats["skipped"]
+            + self.stats["error"]
+            - self.cnt_double_fail_tests
+        )
         logfile.write('<?xml version="1.0" encoding="utf-8"?>')
 
-        logfile.write(Junit.testsuite(
-            self._get_global_properties_node(),
-            [x.to_xml() for x in self.node_reporters_ordered],
-            name=self.suite_name,
-            errors=self.stats['error'],
-            failures=self.stats['failure'],
-            skips=self.stats['skipped'],
-            tests=numtests,
-            time="%.3f" % suite_time_delta, ).unicode(indent=0))
+        logfile.write(
+            Junit.testsuite(
+                self._get_global_properties_node(),
+                [x.to_xml() for x in self.node_reporters_ordered],
+                name=self.suite_name,
+                errors=self.stats["error"],
+                failures=self.stats["failure"],
+                skips=self.stats["skipped"],
+                tests=numtests,
+                time="%.3f" % suite_time_delta,
+            ).unicode(indent=0)
+        )
         logfile.close()
 
     def pytest_terminal_summary(self, terminalreporter):
-        terminalreporter.write_sep("-",
-                                   "generated xml file: %s" % (self.logfile))
+        terminalreporter.write_sep("-", "generated xml file: %s" % (self.logfile))
 
     def add_global_property(self, name, value):
         self.global_properties.append((str(name), bin_xml_escape(value)))
@@ -534,4 +562,4 @@ class LogXML(object):
                     for name, value in self.global_properties
                 ]
             )
-        return ''
+        return ""
